
import InSpice.Logging.Logging as Logging
logger = Logging.setup_logging()

####################################################################################################

from InSpice import Circuit, SubCircuit, SubCircuitFactory
from InSpice.Spice.Parser.HighLevelParser import SpiceSource
from InSpice.Spice.Parser import Translator
from InSpice.Unit import *

raw_spice = '''


EGND 99 0 4 5 1
ENONLIN 100 101 POLY(2) 3 0 4 0 0.0 13.6 0.2 0.005
EAND out1 out0 and(2) in1 0 in2 0 (0.5, 0) (2.8, 3.3)
EGND1 99 0 POLY(2) (3,0) (4,0) 0 .5 .5
ECMP 11 0 TABLE {V(10,9)} = (-5mV, 0V) (5mV, 5V)
E41 4 0 value = {V(3)*V(3)-Offs}
E411 4 0 vol = ’V(3)*V(3)-Offs’

'''

'''
issue with the following lines
ELOPASS 4 0 LAPLACE {V(1)} {5 * (s/100 + 1) / (s^2/42000 + s/60 + 1)} {V(20,21)}= DB (1.000000e+07Hz, 1.633257e-07, -1.859873e+01)

'''


''' Below lines are okay
EGND 99 0 4 5 1
ENONLIN 100 101 POLY(2) 3 0 4 0 0.0 13.6 0.2 0.005
EAND out1 out0 and(2) in1 0 in2 0 (0.5, 0) (2.8, 3.3)
EGND1 99 0 POLY(2) (3,0) (4,0) 0 .5 .5
ECMP 11 0 TABLE {V(10,9)} = (-5mV, 0V) (5mV, 5V)
E41 4 0 value = {V(3)*V(3)-Offs}
E4111 4 0 vol = ’V(3)*V(3)-Offs’


'''


source = raw_spice

spice_source = SpiceSource(source=source, title_line=False)
for obj in spice_source.obj_lines:
    print(obj)
bootstrap_circuit = Translator.Builder().translate(spice_source)
bootstrap_source = str(bootstrap_circuit)
print(bootstrap_source)
# assert bootstrap_source == source

# circuit = Circuit('Test')
# circuit.NonLinearVoltageSource(1, 'output', circuit.gnd,
#                                expression='V(reference, comparator)',
#                                table=((-micro(1), 0),
#                                       (micro(1), 10))
#                                )
# circuit.NonLinearVoltageSource(2, '99', '0', raw_spice='POLY(2) (3,0) (4,0) 0 .5 .5')

# # assert bootstrap_source == source
# print(circuit)


####################################################################################################
#
# InSpice - A Spice Package for Python
# Copyright (C) 2014 Fabrice Salvaire
# Copyright (C) 2025 Innovoltive
# Modified by Innovoltive on April 18, 2025
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
####################################################################################################

__all__ = [
    'TextBuffer',
]

####################################################################################################

import os

####################################################################################################

class TextBuffer:

    ##############################################

    def __init__(self):
        self._lines = []

    ##############################################

    def _append_line(self, line):
        if line is not None:
            _ = str(line)
            if _:
                self._lines.append(_)

    ##############################################

    def __iadd__(self, obj):
        if isinstance(obj, (list, tuple)):
            for _ in obj:
                self._append_line(_)
        else:
            self._append_line(obj)
        return self

    ##############################################

    def __str__(self):
        return os.linesep.join(self._lines)




import InSpice.Logging.Logging as Logging
logger = Logging.setup_logging()

####################################################################################################

from InSpice import Circuit, SubCircuit, SubCircuitFactory
from InSpice.Spice.Parser.HighLevelParser import SpiceSource
from InSpice.Spice.Parser import Translator
from InSpice.Unit import *

class ParallelResistor2(SubCircuit):
    __nodes__ = ('n1', 'n2')
    def __init__(self, name, R1=1@u_Ω, R2=2@u_Ω):
        SubCircuit.__init__(self, name, *self.__nodes__)
        self.R(1, 'n1', 'n2', R1)
        self.R(2, 'n1', 'n2', R2)

circuit = Circuit('Test')
circuit.R('1', 'input', 'n1', 1@u_Ω)
circuit.subcircuit(ParallelResistor2('pr1', R2=2@u_Ω))
circuit.X('1', 'pr1', 1, circuit.gnd)
circuit.subcircuit(ParallelResistor2('pr2', R2=3@u_Ω))
circuit.X('2', 'pr2', 1, circuit.gnd)

source = str(circuit)
print(circuit)

spice_source = SpiceSource(source=source, title_line=False)
bootstrap_circuit = Translator.Builder().translate(spice_source)
bootstrap_source = str(bootstrap_circuit)

print(bootstrap_source)

assert bootstrap_source == source



'''This example shows how to use a single ended to differential amplifier to sense a high voltage
and convert it to a low voltage. The circuit is based on the ADA4940 operational amplifier.
You can see how using AD8137 causes instability in the circuit. The circuit is a simple
'''
####################################################################################################

import numpy as np

import matplotlib.pyplot as plt

####################################################################################################

import InSpice.Logging.Logging as Logging
logger = Logging.setup_logging()

####################################################################################################
from InSpice.Doc.ExampleTools import find_libraries
from InSpice.Plot.BodeDiagram import bode_diagram
from InSpice import SpiceLibrary, Circuit, Simulator, plot
from InSpice.Unit import *

libraries_path = find_libraries()
spice_library = SpiceLibrary(libraries_path)

#f# literal_include('OperationalAmplifier.py')

####################################################################################################

circuit = Circuit('Operational Amplifier')
circuit.include(spice_library['ADA4940'])
circuit.include(spice_library['genopadiff'])
circuit.include(spice_library['LMH6551'])
circuit.include(spice_library['ad8137'])

gain = 80e3/10e6
rh_val = 10e3@u_kΩ
vdd_voltage = 5 
input_offset = 270@u_V
input_amplitude = 1@u_V

circuit.V('V+', 'V+', circuit.gnd, dc_value=vdd_voltage@u_V)
circuit.V('V-', 'V-', circuit.gnd, dc_value=0@u_V)
# place a 0.1uF capacitor from V+ to ground
circuit.C('V+', 'V+', circuit.gnd, 0.1@u_uF)


circuit.SinusoidalVoltageSource('inputh', 'inh', circuit.gnd, offset = input_offset, amplitude=input_amplitude, frequency=1@u_kHz)
circuit.V('inputl', 'inl', circuit.gnd, dc_value=0@u_V)

# ic = 'ad8137'
ic = 'genopadiff'
# ic = 'ada4940'
# ic = 'LMH6551'
if ic == 'ad8137':
    circuit.X('op', 'ad8137', 'in+', 'in-', 'V+', 'V-', 'out+', 'out-', 'Vcm')
elif ic == 'genopadiff':
    circuit.X('op', 'genopadiff', 'in+', 'in-', 'V+', 'V-', 'out+', 'out-', 'Vcm')

elif ic == 'ada4940':
    circuit.X('op', 'ADA4940', 'FB-', 'FB+', 'in+', 'in-', 'V+', 'V-','out+', 'out-', 'Vcm')
elif ic == 'LMH6551':
    # Note that the LMH6551 needs a Vcm voltage to work properly. This is the common mode voltage.
    circuit.X('op', 'LMH6551', 'in+', 'in-', 'V+', 'V-', 'Vcm', 'out+', 'out-')
    circuit.V('Vcm', 'Vcm', circuit.gnd, dc_value=vdd_voltage@u_V)
else:
    raise ValueError('Unknown IC: {}'.format(ic))

circuit.R('inph', 'inh',   'in+', rh_val)
circuit.R('innh', 'inl',   'in-', rh_val)

circuit.R('innl', 'in+', 'out-', gain*rh_val)
circuit.R('inpl', 'in-', 'out+', gain*rh_val)

# put a 56 omhm in series with each output of th opamp
circuit.R('out+', 'out+', 'outp', 56@u_Ω)
circuit.R('out-', 'out-', 'outn', 56@u_Ω)
# place 39pf from outp and outn to ground
# circuit.C('outp', 'outp', circuit.gnd, 390@u_pF)
# circuit.C('outn', 'outn', circuit.gnd, 390@u_pF)

circuit.R('load', 'outp', 'outn', 470@u_Ω)
# place a 0.22uF capacitor from Vcm to ground
circuit.C('Vcm', 'Vcm', circuit.gnd, 0.22@u_uF)



simulator = Simulator.factory()
simulation = simulator.simulation(circuit, temperature=25, nominal_temperature=25)
simulation.options(rshunt=1e12)
simulation.options(max_step=1e-6)
analysis = simulation.transient(step_time=1e-6, end_time=10e-3, start_time=0@u_ms, max_time=1@u_us, use_initial_condition=False)

figure, (ax1, ax2) = plt.subplots(2, figsize=(20, 10))

plt.title("output Operational Amplifier")
plt.xlabel("Time (s)")
plt.ylabel("Voltage (V)")
ax1.plot(analysis.time, analysis['inh'], label='input voltage')

# add legend
ax1.legend()
ax2.plot(analysis.time, analysis['outp']-analysis['outn'], label='differential output')

ax2.legend()

plt.tight_layout()
plt.show()

#f# save_figure('figure', 'operational-amplifier.png')
